const common = require('./webpack.config.common');
const { resolve } = require('path');
const webpack = require('webpack');
const HtmlWebpackPlugin = require('html-webpack-plugin');
const MiniCssExtractPlugin = require('mini-css-extract-plugin');
const InterpolateHtmlPlugin = require('./InterpolateHtmlPlugin');
const ScriptExtHtmlWebpackPlugin = require("script-ext-html-webpack-plugin");
const TerserPlugin = require('terser-webpack-plugin');
const path = require("path");
const envfile = require("envfile");
const env = envfile.parseFileSync(path.resolve("", ".env"));
const checkForLegacyErrors = require("./check-for-legacy-errors");

require('dotenv').config();

const { page, scenario, strategy, scenariosConfig, service, defaultService, defaultScenario } = env;
const publicPath = process.env.publicPath || ''

const commitHash = require('child_process')
  .execSync('git rev-parse --short HEAD')
  .toString();

const designer = require('child_process')
  .execSync('git config --global user.name')
  .toString();

checkForLegacyErrors();
module.exports = {
  mode: 'production',
  entry: {
    main: resolve('src/index')
  },
  externals: common.externals,
  output: {
    filename: `[name].[chunkhash].js`,
    path: resolve('dist'),
    publicPath: '',
    assetModuleFilename: common.assetModuleFilename
  },
  resolve: common.resolve,
  devtool: 'source-map',
  module: {
    rules: [
      common.modules.purs,
      common.modules.ts,
      {
        test: /\.styl$/,
        oneOf: [{
            resourceQuery: /^\?raw$/,
            use: [
              MiniCssExtractPlugin.loader,
              'css-loader',
              common.loaders.postcss,
              common.loaders.stylus,
            ]
          },
          {
            use: [
              MiniCssExtractPlugin.loader,
              common.loaders["typings-for-css-camelCase"],
              common.loaders.postcss,
              common.loaders.stylus,
            ]
          }
        ]
      },
      {
        test: /\.(scss|sass)$/,
        oneOf: [{
            resourceQuery: /^\?raw$/,
            use: [
              MiniCssExtractPlugin.loader,
              'css-loader',
              common.loaders.postcss,
              common.loaders.sass,
            ]
          },
          {
            use: [
              MiniCssExtractPlugin.loader,
              'css-loader',
              common.loaders.postcss,
              common.loaders.sass,
            ]
          }
        ]
      },
      {
        test: /\.(css|less)$/,
        oneOf: [{
            resourceQuery: /^\?raw$/,
            use: [
              MiniCssExtractPlugin.loader,
              'css-loader',
              common.loaders.postcss,
              common.loaders.less,
            ]
          },
          {
            resourceQuery: /^\?dash-case$/,
            use: [
              MiniCssExtractPlugin.loader,
              common.loaders["typings-for-css"],
              common.loaders.postcss,
              common.loaders.less,
            ]
          },
          {
            use: [
              MiniCssExtractPlugin.loader,
              common.loaders["typings-for-css-camelCase"],
              common.loaders.postcss,
              common.loaders.less,
            ]
          }
        ]
      },
      common.modules.url,
    ],
  },
  optimization: {
    minimizer: [
      new TerserPlugin({
        terserOptions: {
          // Terser specific options here
        }
      })
    ]
  },
  plugins: [
    new webpack.WatchIgnorePlugin({
      paths: [
        /styl\.d\.ts$/,
        /css\.d\.ts$/
      ]
    }),
    common.plugins.define,
    new MiniCssExtractPlugin({
      filename: `[name].[contenthash:8].css`,
      chunkFilename: `[name].[contenthash:8].chunk.css`,
    }),
    new HtmlWebpackPlugin({
      title: process.env.title,
      show_social_meta: process.env.show_social_meta,
      og_url: process.env.og_url,
      og_type: process.env.og_type,
      og_title: process.env.og_title,
      og_image: process.env.og_image,
      og_description: process.env.og_description,
      page: page,
      scenario: scenario,
      strategy: strategy,
      scenariosConfig: scenariosConfig,
      service: service,
      defaultService: defaultService,
      defaultScenario: defaultScenario,
      designer: designer.replace("\n", "").split(' ')[0],
      commitHash: commitHash.replace("\n", ""),
      date: Date(),
      inject: true,
      template: resolve(`src/index.html`),
      filename: `index.html`,
      minify: {
        removeComments: true,
        collapseWhitespace: true,
        removeRedundantAttributes: true,
        useShortDoctype: true,
        removeEmptyAttributes: true,
        removeStyleLinkTypeAttributes: true,
        keepClosingSlash: true,
        minifyJS: true,
        minifyCSS: true,
        minifyURLs: true,
      },
    }),
    new ScriptExtHtmlWebpackPlugin({
      defaultAttribute: "defer"
    }),
    new InterpolateHtmlPlugin({
      'PUBLIC_URL': publicPath + "/static/" + page
    }),
    // ... other plugins
  ].filter((v) => v !== null),
};
