const common = require("./webpack.config.common");
const { resolve } = require("path");
const webpack = require("webpack");
const HtmlWebpackPlugin = require("html-webpack-plugin");
const MiniCssExtractPlugin = require("mini-css-extract-plugin");
const InterpolateHtmlPlugin = require("./InterpolateHtmlPlugin");
const ScriptExtHtmlWebpackPlugin = require("script-ext-html-webpack-plugin");
const TerserPlugin = require("terser-webpack-plugin");
const path = require("path");
const envfile = require("envfile");
const env = envfile.parseFileSync(path.resolve("", ".env"));
const checkForLegacyErrors = require("./check-for-legacy-errors");

require("dotenv").config();

const {
  page,
  scenario,
  strategy,
  scenariosConfig,
  service,
  defaultService,
  defaultScenario,
} = env;
const HTML = "true" == process.env.html || "true" == process.env.HTML;
const TNC = "true" == process.env.tnc || "true" == process.env.TNC;
const publicPath = process.env.publicPath || "";

const commitHash = require("child_process")
  .execSync("git rev-parse --short HEAD")
  .toString();

const designer = require("child_process")
  .execSync("git config --global user.name")
  .toString();

checkForLegacyErrors();
module.exports = {
  mode: "production",
  entry: {
    main: resolve("src/index"),
    // !!page && page != "default"
    // ? resolve(__dirname, `../src/landing-pages/${page}/index`)
    // : resolve(__dirname, '../src'),
    // main: resolve(__dirname, '../src'),
    // vendor: [
    //   'react-redux',
    //   'react-router-dom',
    //   'redux',
    //   'redux-thunk',
    //   'styled-components',
    // ],
  },
  externals: common.externals,
  output: {
    filename: `static/${page}/js/[name].[chunkhash].js`,
    path: resolve("dist"),
    publicPath: publicPath + "/",
    assetModuleFilename: common.assetModuleFilename,
  },
  resolve: common.resolve,
  devtool: "source-map",
  module: {
    rules: [
      common.modules.purs,
      common.modules.ts,
      // HTML && common.modules.html || {},
      {
        test: /\.styl$/,
        oneOf: [
          {
            resourceQuery: /^\?raw$/,
            use: [
              MiniCssExtractPlugin.loader,
              // {
              // loader: require.resolve('css-loader'),
              // options: {
              //   modules: true,
              //   localIdentName: '[path][name]__[local]--[hash:base64:5]'
              // }
              // },
              "css-loader",
              common.loaders.postcss,
              common.loaders.stylus,
            ],
          },
          {
            use: [
              MiniCssExtractPlugin.loader,
              common.loaders["typings-for-css-camelCase"],
              common.loaders.postcss,
              common.loaders.stylus,
            ],
          },
        ],
      },
      {
        test: /\.(scss|sass)$/,
        oneOf: [
          {
            resourceQuery: /^\?raw$/,
            use: [
              MiniCssExtractPlugin.loader,
              "css-loader",
              common.loaders.postcss,
              common.loaders.sass,
            ],
          },
          {
            use: [
              MiniCssExtractPlugin.loader,
              "css-loader",
              common.loaders.postcss,
              common.loaders.sass,
            ],
          },
        ],
      },
      {
        test: /\.(css|less)$/,
        // exclude: /node_modules/,
        oneOf: [
          {
            resourceQuery: /^\?raw$/,
            use: [
              MiniCssExtractPlugin.loader,
              "css-loader",
              common.loaders.postcss,
              common.loaders.less,
            ],
          },
          {
            resourceQuery: /^\?dash-case$/,
            use: [
              MiniCssExtractPlugin.loader,
              common.loaders["typings-for-css"],
              common.loaders.postcss,
              common.loaders.less,
            ],
          },
          {
            use: [
              MiniCssExtractPlugin.loader,
              common.loaders["typings-for-css-camelCase"],
              common.loaders.postcss,
              common.loaders.less,
            ],
          },
        ],
      },
      common.modules.url,
    ],
  },
  optimization: {
    usedExports: true,
    splitChunks: {
      chunks: "all",
      minSize: 50000,
      cacheGroups: {
        vendors: {
          test: /[\\/]node_modules[\\/](?!ouisys-engine)/,
          name: "vendors",
          chunks: "all",
          priority: -10,
          reuseExistingChunk: true,
        },
        coreJs: {
          test: /[\\/]node_modules[\\/](core-js)[\\/]/,
          name: "core-js",
          chunks: "all",
          priority: 30,
        },
        pacmanClient: {
          test: /[\\/]node_modules[\\/](pacman-client)[\\/]/,
          name: "pacman-client",
          chunks: "all",
          priority: 30,
        },
        largeLibs: {
          test(module, chunks) {
            const size = module.size();
            const context = module.context || "";
            const isNodeModules = context.includes("node_modules");
            return isNodeModules && size > 50000;
          },
          name(module) {
            const packageName = module.context.match(
              /[\\/]node_modules[\\/](.*?)([\\/]|$)/
            )[1];
            return `${packageName.replace("@", "")}`;
          },
          priority: 10,
          reuseExistingChunk: true,
        },
      },
    },

    minimize: true,
    minimizer: [new TerserPlugin({
        terserOptions: {
          format: {
            comments: false,
          },
        },
        extractComments: false,
      })],
  },
  plugins: [
    new webpack.WatchIgnorePlugin({
      paths: [/styl\.d\.ts$/, /css\.d\.ts$/],
    }),
    common.plugins.define,
    common.plugins.favicon,
    new MiniCssExtractPlugin({
      // Options similar to the same options in webpackOptions.output
      // both options are optional
      filename: `static/${page}/css/[name].[contenthash:8].css`,
      chunkFilename: `static/${page}/css/[name].[contenthash:8].chunk.css`,
    }),
    new HtmlWebpackPlugin({
      title: process.env.title,
      show_social_meta: process.env.show_social_meta,
      og_url: process.env.og_url,
      og_type: process.env.og_type,
      og_title: process.env.og_title,
      og_image: process.env.og_image,
      og_description: process.env.og_description,
      page: page,
      scenario: scenario,
      strategy: strategy,
      scenariosConfig: scenariosConfig,
      service: service,
      defaultService,
      defaultScenario,
      designer: designer.replace("\n", "").split(" ")[0],
      commitHash: commitHash.replace("\n", ""),
      date: Date(),
      inject: true,
      template: HTML
        ? resolve(`src/index.html`)
        : resolve(__dirname, "template.ssr.html"),
      filename: TNC
        ? `static/${page}/html/index.ssr.html`
        : `static/${page}/html/index.html`,
      minify: {
        removeComments: true,
        collapseWhitespace: true,
        removeRedundantAttributes: true,
        useShortDoctype: true,
        removeEmptyAttributes: true,
        removeStyleLinkTypeAttributes: true,
        keepClosingSlash: true,
        minifyJS: true,
        minifyCSS: true,
        minifyURLs: true,
      },
    }),
    new ScriptExtHtmlWebpackPlugin({
      defaultAttribute: "defer",
      // inline: [/js/ig]
    }),
    new InterpolateHtmlPlugin({
      PUBLIC_URL: publicPath + "/static/" + page,
    }),
  ].filter((v) => v !== null),
};
